using BPA.KitChen.GroupMeal.SqlSugar;
using BPA.SAAS.Manage.Application.Device.Dtos.Device;
using BPA.SAAS.Manage.Comm.Model;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.System;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.Device.Interface
{
    public interface IDeviceService
    {
        Task<PageUtil> GetDeviceInfoPage(DeviceQueryInputDto inputDto);
        Task<List<ListSelectDeviceQuery>> GetDeviceList();
        Task<List<ListSelectDeviceQuery>> GetDeviceByStopList(string stopId);
        Task<bool> AddDevice(DeviceInfoBaseDto inputDto);
        Task<bool> UpdateDevice(DeviceInfoBaseDto inputDto);
        Task<bool> DelDeviceInfo(List<string> inputList);
        Task<bool> AddDeviceTypeAsync(DeviceTypeBaseDto inputDto);
        Task<List<ListSelectQuery>> GetDeviceTypeList();

    }
}
